<table border="1" cellpadding="0" cellspacing="0" class="linkBox">
	<tbody>
		<tr>
			<td class="linkBox" ng-show="params.reporttype == 'hosts'">
				<a href="trends.html?cgiurl={{params.cgiurl | uriComponentEncode}}&host={{params.host | uriComponentEncode}}&t1={{params.t1}}&t2={{params.t2}}&assumestateretention={{params.assumeStateRetention ? 'yes' : 'no'}}" target="_self">
						View Trends For This Host
				</a>
				<br>
				<a href="{{params.cgiurl}}avail.cgi?host={{params.host | uriComponentEncode}}&t1={{params.t1}}&t2={{params.t2}}&assumestateretention={{params.assumeStateRetention ? 'yes' : 'no'}}&show_log_entries" target="_self">
					View Availability Report For This Host
				</a>
				<br>
				<a href="{{params.cgiurl}}status.cgi?host={{params.host | uriComponentEncode}}" target="_self">
					View Status Detail For This Host
				</a>
				<br>
				<a href="{{params.cgiurl}}history.cgi?host={{aparams.host | uriComponentEncode}}" target="_self">
					View History For This Host
				</a>
				<br>
				<a href="{{params.cgiurl}}notifications.cgi?host={{params.host | uriComponentEncode}}" target="_self">
					View Notifications For This Host
				</a>
				<br>
			</td>
			<td class="linkBox" ng-show="params.reporttype == 'services'">
				<a href="trends.html?cgiurl={{params.cgiurl | uriComponentEncode}}&host={{params.host | uriComponentEncode}}&service={{params.service | uriComponentEncode}}&t1={{params.t1}}&t2={{params.t2}}&assumestateretention={{params.assumeStateRetention ? 'yes' : 'no'}}" target="_self">
					View Trends For This Service
				</a>
				<br>
				<a href="{{params.cgiurl}}avail.cgi?host={{params.host | uriComponentEncode}}&service={{params.service | uriComponentEncode}}&t1={{params.t1}}&t2={{params.t2}}&assumestateretention={{params.assumeStateRetention ? 'yes' : 'no'}}&show_log_entries" target="_self">
					View Availability Report For This Service
				</a>
				<br>
				<a href="{{params.cgiurl}}history.cgi?host={{params.host | uriComponentEncode}}&service={{params.service | uriComponentEncode}}" target="_self">
					View History For This Service
				</a>
				<br>
				<a href="{{params.cgiurl}}notifications.cgi?host={{params.host | uriComponentEncode}}&service={{params.service | uriComponentEncode}}" target="_self">
					View Notifications For This Service
				</a>
				<br>
			</td>
		</tr>
	</tbody>
</table>

