/* exfoliation: a nagios makeover                                          */
/* Copyright 2010 Matthew Wall, all rights reserved                        */
/*                                                                         */
/* Permission to use, copy, modify, and distribute this software for any   */
/* purpose with or without fee is hereby granted, provided that the above  */
/* copyright notice and this permission notice appear in all copies.       */

.status { }

.itemTotalsTitle { font-size: 8pt; font-style: italic; clear:both;}

table.status { font-size: 9pt; padding: 0 0 10 0; }
th.status    { font-size: 9pt; text-align: left; padding: 0 3px 0 3px; border-bottom: 1px solid #777777; color: #333333; }
div.status   { font-size: 10pt; text-align: center; }
.statusOdd   { font-size: 8pt; background-color: #e7e7e7; line-height: 150%; padding: 0 4 0 4; }
.statusEven  { font-size: 8pt; background-color: #f4f2f2; line-height: 150%; padding: 0 4 0 4; }

.statusPENDING  { font-size: 8pt; background-color: #acacac; border: 1px solid #777777; padding: 0 5 0 5; }
.statusOK       { font-size: 8pt; background-color: #88d066; border: 1px solid #777777; padding: 0 5 0 5; }
.statusRECOVERY { font-size: 8pt; background-color: #88d066; border: 1px solid #777777; padding: 0 5 0 5; }
.statusUNKNOWN  { font-size: 8pt; background-color: #ffbb55; border: 1px solid #777777; padding: 0 5 0 5; }
.statusWARNING  { font-size: 8pt; background-color: #ffff00; border: 1px solid #777777; padding: 0 5 0 5; }
.statusCRITICAL { font-size: 8pt; background-color: #f88888; border: 1px solid #777777; padding: 0 5 0 5; }
.statusCRITICALACK { font-size: 8pt; background-color: #f88888; border: 1px solid #777777; padding: 0 5 0 5; }

.statusHOSTPENDING          { font-size: 8pt; background-color: #acacac; line-height: 150%; padding: 0 4 0 4; }
.statusHOSTUP               { font-size: 8pt; background-color: #cce8cc; line-height: 150%; padding: 0 4 0 4; }
.statusHOSTDOWN             { font-size: 8pt; background-color: #ffdddd; line-height: 150%; padding: 0 4 0 4; }
.statusHOSTDOWNACK          { font-size: 8pt; background-color: #ffdddd; line-height: 150%; padding: 0 4 0 4; }
.statusHOSTDOWNSCHED        { font-size: 8pt; background-color: #ffdddd; line-height: 150%; padding: 0 4 0 4; }
.statusHOSTUNREACHABLE      { font-size: 8pt; background-color: #ffddaa; line-height: 150%; padding: 0 4 0 4; }
.statusHOSTUNREACHABLEACK   { font-size: 8pt; background-color: #ffddaa; line-height: 150%; padding: 0 4 0 4; }
.statusHOSTUNREACHABLESCHED { font-size: 8pt; background-color: #ffddaa; line-height: 150%; padding: 0 4 0 4; }

.statusBGUNKNOWN          { font-size: 8pt; background-color: #ffddaa; }
.statusBGUNKNOWNACK       { font-size: 8pt; background-color: #ffddaa; }
.statusBGUNKNOWNSCHED     { font-size: 8pt; background-color: #ffddaa; }
.statusBGWARNING          { font-size: 8pt; background-color: #feffc1; }
.statusBGWARNINGACK       { font-size: 8pt; background-color: #feffc1; }
.statusBGWARNINGSCHED     { font-size: 8pt; background-color: #feffc1; }
.statusBGCRITICAL         { font-size: 8pt; background-color: #ffdddd; }
.statusBGCRITICALACK      { font-size: 8pt; background-color: #ffdddd; }
.statusBGCRITICALSCHED    { font-size: 8pt; background-color: #ffdddd; }
.statusBGDOWN             { font-size: 8pt; background-color: #ffdddd; }
.statusBGDOWNACK          { font-size: 8pt; background-color: #ffdddd; }
.statusBGDOWNSCHED        { font-size: 8pt; background-color: #ffdddd; }
.statusBGUNREACHABLE      { font-size: 8pt; background-color: #ffddaa; }
.statusBGUNREACHABLEACK   { font-size: 8pt; background-color: #ffddaa; }
.statusBGUNREACHABLESCHED { font-size: 8pt; background-color: #ffddaa; }

div.serviceTotals      { font-size: 10pt; text-align: center; font-weight: bold; }
table.serviceTotals    { font-size: 10pt; font-weight: bold; }
th.serviceTotals,a.serviceTotals { font-size: 8pt; }
td.serviceTotals       { font-size: 8pt; text-align: center; background-color: #e0e0e0; border: 1px solid #777777; padding: 2 4 2 4; }
.serviceTotalsOK       { font-size: 8pt; text-align: center; background-color: #88d066; border: 1px solid #777777; padding: 2 4 2 4; }
.serviceTotalsWARNING  { font-size: 8pt; text-align: center; background-color: #ffff00; border: 1px solid #777777; padding: 2 4 2 4; }
.serviceTotalsUNKNOWN  { font-size: 8pt; text-align: center; background-color: #ffbb55; border: 1px solid #777777; padding: 2 4 2 4; }
.serviceTotalsCRITICAL { font-size: 8pt; text-align: center; background-color: #f88888; border: 1px solid #777777; padding: 2 4 2 4; }
.serviceTotalsPENDING  { font-size: 8pt; text-align: center; background-color: #acacac; border: 1px solid #777777; padding: 2 4 2 4; }
.serviceTotalsPROBLEMS { font-size: 8pt; text-align: center; background-color: #aaccff; border: 1px solid #777777; padding: 2 4 2 4; }

div.hostTotals         { font-size: 10pt; text-align: center; font-weight: bold; }
table.hostTotals       { font-size: 10pt; font-weight: bold; }
th.hostTotals,a.hostTotals { font-size: 8pt; }
td.hostTotals          { font-size: 8pt; text-align: center; background-color: #e0e0e0; border: 1px solid #777777; padding: 2 4 2 4; }
.hostTotalsUP          { font-size: 8pt; text-align: center; background-color: #88d066; border: 1px solid #777777; padding: 2 4 2 4; }
.hostTotalsDOWN        { font-size: 8pt; text-align: center; background-color: #f88888; border: 1px solid #777777; padding: 2 4 2 4; }
.hostTotalsUNREACHABLE { font-size: 8pt; text-align: center; background-color: #ffbb55; border: 1px solid #777777; padding: 2 4 2 4; }
.hostTotalsPENDING     { font-size: 8pt; text-align: center; background-color: #acacac; border: 1px solid #777777; padding: 2 4 2 4; }
.hostTotalsPROBLEMS    { font-size: 8pt; text-align: center; background-color: #aaccff; border: 1px solid #777777; padding: 2 4 2 4; }

.miniStatusPENDING     { font-size: 8pt; text-align: center; background-color: #acacac; border: 1px solid #777777; padding: 0 5 0 5; }
.miniStatusOK          { font-size: 8pt; text-align: center; background-color: #88d066; border: 1px solid #777777; padding: 0 5 0 5; }
.miniStatusUNKNOWN     { font-size: 8pt; text-align: center; background-color: #ffbb55; border: 1px solid #777777; padding: 0 5 0 5; }
.miniStatusWARNING     { font-size: 8pt; text-align: center; background-color: #ffff00; border: 1px solid #777777; padding: 0 5 0 5; }
.miniStatusCRITICAL    { font-size: 8pt; text-align: center; background-color: #f88888; border: 1px solid #777777; padding: 0 5 0 5; }

.miniStatusUP          { font-size: 8pt; text-align: center; background-color: #88d066; border: 1px solid #777777; padding: 0 5 0 5; }
.miniStatusDOWN        { font-size: 8pt; text-align: center; background-color: #f88888; border: 1px solid #777777; padding: 0 5 0 5; }
.miniStatusUNREACHABLE { font-size: 8pt; text-align: center; background-color: #ffbb55; border: 1px solid #777777; padding: 0 5 0 5; }

/* page number styles, added 2/01/2012 -MG */
#top_page_numbers { float:right;} 
#result_limit { display:inline;} 
.pagenumber { display: block; float:left; border: 1px solid #AAAAAA; padding: 0 2px 0 2px; margin: 1px;text-align:center; height:15px; }
a.pagenumber:hover { background-color: #EFEFEF;text-decoration:none;}
.current_page { color: #AAA; }
#inner_numbers { clear:right;} 
#pagelimit,#bottom_page_numbers { font-size:8pt;}
