/* exfoliation: a nagios makeover                                          */
/* Copyright 2010 Matthew Wall, all rights reserved                        */
/*                                                                         */
/* Permission to use, copy, modify, and distribute this software for any   */
/* purpose with or without fee is hereby granted, provided that the above  */
/* copyright notice and this permission notice appear in all copies.       */

.summary { }

/* override to match filter table style */
.optBoxItem         { font-size: 8pt; font-weight: bold; }
.optBoxValue        { font-size: 8pt; }

/* override to match query info style */
.dataSubTitle { font-size: 8pt; text-align: center; font-weight: normal; }

/* override so we get a bit of whitespace */
table.data { padding-top: 15; }

.reportDataOdd  { font-size: 9pt; background-color: #e7e7e7; padding: 0 4 0 4; }
.reportDataEven { font-size: 9pt; background-color: #f4f2f2; padding: 0 4 0 4; }

.hostUP          { font-size: 9pt; background-color: #88d066; border: 1px solid #777777; padding: 0 4 0 4; }
.hostDOWN        { font-size: 9pt; background-color: #f88888; border: 1px solid #777777; padding: 0 4 0 4; }
.hostUNREACHABLE { font-size: 9pt; background-color: #ffbb55; border: 1px solid #777777; padding: 0 4 0 4; }

.serviceOK       { font-size: 9pt; background-color: #88d066; border: 1px solid #777777; padding: 0 4 0 4; }
.serviceWARNING  { font-size: 9pt; background-color: #ffff00; border: 1px solid #777777; padding: 0 4 0 4; }
.serviceUNKNOWN  { font-size: 9pt; background-color: #ffbb55; border: 1px solid #777777; padding: 0 4 0 4; }
.serviceCRITICAL { font-size: 9pt; background-color: #f88888; border: 1px solid #777777; padding: 0 4 0 4; }
